#define UNICODE 1

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <iostream.h>
#include "packet32.h"

typedef unsigned int __u32;

WCHAR* SChar2WChar(char* nome)
{
	int i;
	WCHAR* TmpName;

	TmpName=(WCHAR*) malloc ((strlen(nome)+2)*sizeof(WCHAR));
	for (i=0;i<(signed)strlen(nome)+1; i++)
		TmpName[i]=nome[i];
	TmpName[i]=0;
	return TmpName;
}

//---------------------------------------------------------------------------
/* Convert a Unicode dotted-quad to a 32-bit IP address.
   Doesn't check to make sure it's valid. */

ULONG inet_addrU(const WCHAR *cp)
{
	ULONG val, part;
	WCHAR c;
	int i;

	val = 0;
	for (i = 0; i < 4; i++) {
		part = 0;
		while ((c = *cp++) != '\0' && c != '.') {
			if (c < '0' || c > '9')
				return -1;
			part = part*10 + (c - '0');
		}
		if (part > 255)
			return -1;	
		val = (val << 8) | part;
		if (i == 3) {
			if (c != '\0')
				return -1;	// extra gunk at end of string 
		} else {
			if (c == '\0')
				return -1;	// string ends early 
		}
	}
	return val;
}


BOOLEAN PacketGetNetInfoEx(
	LPTSTR AdapterName, PULONG netp, PULONG maskp, PULONG defp, bool *dhcpFlag)
{
	char	*AdapterNameA;
	WCHAR	*AdapterNameU;
	WCHAR	*ifname;
	HKEY	SystemKey;
	HKEY	ControlSetKey;
	HKEY	ServicesKey;
	HKEY	InterfaceKey;
	HKEY	ParametersKey;
	HKEY	TcpIpKey;
	LONG	status;
	WCHAR	String[1024+1];
	DWORD	RegType;
	ULONG	BufLen;
	DWORD	DHCPEnabled;

	*dhcpFlag = false;

	AdapterNameA = (char*)AdapterName;
	if(AdapterNameA[1] != 0) {	//ASCII
		AdapterNameU = SChar2WChar(AdapterNameA);
		AdapterName = AdapterNameU;
	} else {				//Unicode
		AdapterNameU = NULL;
	}

	ifname = wcsrchr(AdapterName, '\\');
	if (ifname == NULL)
		ifname = AdapterName;
	else
		ifname++;
	
	if (wcsncmp(ifname, L"Packet_", 7) == 0)
		ifname += 7;

	status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,TEXT("SYSTEM"),0,KEY_READ,&SystemKey);
	if (status != ERROR_SUCCESS)
		goto fail;
	
	status = RegOpenKeyEx(SystemKey,TEXT("CurrentControlSet"),0,KEY_READ,&ControlSetKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(SystemKey);
		goto fail;
	}
	RegCloseKey(SystemKey);

	status = RegOpenKeyEx(ControlSetKey,TEXT("Services"),0,KEY_READ,&ServicesKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(ControlSetKey);
		goto fail;
	}
	RegCloseKey(ControlSetKey);

	status = RegOpenKeyEx(ServicesKey,ifname,0,KEY_READ,&InterfaceKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(ServicesKey);
		goto fail;
	}
	RegCloseKey(ServicesKey);
	
	status = RegOpenKeyEx(InterfaceKey,TEXT("Parameters"),0,KEY_READ,&ParametersKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(InterfaceKey);
		goto fail;
	}
	RegCloseKey(InterfaceKey);
	
	status = RegOpenKeyEx(ParametersKey,TEXT("TcpIp"),0,KEY_READ,&TcpIpKey);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(ParametersKey);
		goto fail;
	}
	RegCloseKey(ParametersKey);

	// IP Address
	BufLen = sizeof String;
	status = RegQueryValueEx(TcpIpKey,TEXT("IPAddress"),NULL,&RegType,(LPBYTE)String,&BufLen);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(TcpIpKey);
		goto fail;
	}
	*netp = inet_addrU(String);

	// DefaultGateway
	BufLen = sizeof String;	
	status = RegQueryValueEx(TcpIpKey,TEXT("DefaultGateway"),NULL,&RegType,(LPBYTE)String,&BufLen);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(TcpIpKey);
		goto fail;
	}
	*defp = inet_addrU(String);

	// SubnetMask
	BufLen = sizeof String;
	status=RegQueryValueEx(TcpIpKey,TEXT("SubnetMask"),NULL,&RegType,(LPBYTE)String,&BufLen);
	if (status != ERROR_SUCCESS) {
		RegCloseKey(TcpIpKey);
		goto fail;
	}
	*maskp = inet_addrU(String);

	/* See if DHCP is used by this system */
	BufLen = 4;
	status=RegQueryValueEx(TcpIpKey,TEXT("EnableDHCP"),NULL,&RegType,(LPBYTE)&DHCPEnabled,&BufLen);
	if (status == ERROR_SUCCESS) {

		/* Retrieve the netmask */
		if(DHCPEnabled){
		
			*dhcpFlag = true;

			// DHCP IP Address
			BufLen = sizeof String;
			status=RegQueryValueEx(TcpIpKey,TEXT("DhcpIPAddress"),NULL,&RegType,
				(LPBYTE)String,&BufLen);
			if (status != ERROR_SUCCESS) {
				RegCloseKey(TcpIpKey);
				goto fail;
			}

			*netp = inet_addrU(String);
			
			// DHCP DefaultGateway
			BufLen = sizeof String;
			status=RegQueryValueEx(TcpIpKey,TEXT("DhcpDefaultGateway"),NULL,&RegType,
				(LPBYTE)String,&BufLen);
			if (status != ERROR_SUCCESS) {
				RegCloseKey(TcpIpKey);
				goto fail;
			}

			*defp = inet_addrU(String);
			
			// DHCP SubnetMask
			BufLen = sizeof String;
			status=RegQueryValueEx(TcpIpKey,TEXT("DHCPSubnetMask"),NULL,&RegType,
				(LPBYTE)String,&BufLen);

			if (status != ERROR_SUCCESS) {
				RegCloseKey(TcpIpKey);
				goto fail;
			}

			*maskp = inet_addrU(String);			
		}
	}

	if (AdapterNameU != NULL)
		free(AdapterNameU);
	return TRUE;


fail:
	if (AdapterNameU != NULL)
		free(AdapterNameU);
    return FALSE;
}

void main()
{
	__u32			ipAddr;				// IP Address
	__u32			maskAddr;			// Mask (Subnet) Address
	__u32			gwAddr;

	// string that contains a list of the network adapters
	WCHAR		TempBuffer[1024]; 

	char        adapterList[10][512];
	char        adapterList2[10][512];

	DWORD dwVersion=GetVersion();
	DWORD dwWindowsMajorVersion =  (DWORD)(LOBYTE(LOWORD(dwVersion)));

	// Windows NT 
	if (!(!(dwVersion >= 0x80000000 && dwWindowsMajorVersion >= 4)))
		return;
  
	ULONG		AdapterLength = 1024;

	memset(TempBuffer,0, sizeof(TempBuffer));
	memset(adapterList,0, sizeof(adapterList));
	memset(adapterList2,0, sizeof(adapterList2));
	
//	PacketGetAdapterNames((char *)TempBuffer,&AdapterLength);
	PacketGetAdapterNames((unsigned short *)TempBuffer,&AdapterLength);

	WCHAR		*temp,*temp1;

	temp=TempBuffer;
	temp1=TempBuffer;
	
	int i = 0;

	while ((*temp!='\0')||(*(temp-1)!='\0'))
	{
		if (*temp=='\0') 
		{
			memcpy(adapterList[i],temp1,(temp-temp1)*2);
			temp1=temp+1;
			i++;
		}
	
		temp++;
	}
	 
	// Change adapterList the part of "/" to "//" for Windows NT
	int AdapterNum = i;
	
	for (i=0;i<AdapterNum;i++)
	{

		memcpy(adapterList2[i],adapterList[i],sizeof(adapterList[i]));
		temp1 = temp = (WCHAR *)(adapterList2[i]);
		
		int adapterList2Length = 0;
		while ((*temp!='\0')||(*(temp-1)!='\0'))
		{
			adapterList2Length++;
			temp++;
		}

		int j =0;
		temp = temp1;
		while ((*temp!='\0')||(*(temp-1)!='\0'))
		{
			if (*temp == 0x005c)
			{
				int k;
				for (k = adapterList2Length ; k > j ; k--)
				{
					*(temp1 + k + 1) = *(temp1 + k); 
				}

				*(temp + 1) =  0x005c;	// add "/"
				temp++;
				adapterList2Length++;
			}
			j++;
			temp++;
		}
	}

	for (i=0;i<AdapterNum;i++)
	{
		// Get the IP Address
		ipAddr = maskAddr = gwAddr = 0;

		bool dhcpFlag;

		BOOLEAN truefalse = 
			PacketGetNetInfoEx((LPTSTR)adapterList2[i],
			(PULONG)(&ipAddr),(PULONG)(&maskAddr),(PULONG)(&gwAddr), &dhcpFlag);

		char dotIP[16];
		memset(dotIP, 0, 16);
		struct in_addr	addr;

		addr.s_addr = ntohl(ipAddr);
		memcpy(dotIP, inet_ntoa(addr), 16); 

		//wprintf(L"\n%d- %s",i+1,adapterList2[i]);
		wprintf(L"\n%d- %s",i+1,adapterList[i]);
		if (dhcpFlag) 
			printf("(DHCP)\n");
		else
			printf("\n");

		printf("        IP Address. . . . . . . . . : %s\n",dotIP);

		addr.s_addr = ntohl(maskAddr);
		memcpy(dotIP, inet_ntoa(addr), 16); 
		printf("        Subnet Mask . . . . . . . . : %s\n",dotIP);

		if (gwAddr != 0xffffffff)
		{
			addr.s_addr = ntohl(gwAddr);
			memcpy(dotIP, inet_ntoa(addr), 16); 
			printf("        Default Gateway . . . . . . : %s\n",dotIP);	
		}

		printf("\n");

	}

	int Open;

	while(1)
	{
		printf("\n[Step.1] Select the adapter number for Outer Network: ");
	
		scanf("%d",&Open);

		if (Open>AdapterNum) 
		{
			printf("\nThe number must be smaller than %d\n",AdapterNum);
			continue;
		}
		
		if (Open <= 0)
		{
			printf("\nTerminated\n");
			exit(0);
		}

		bool dhcpFlag;
		BOOLEAN truefalse = 
			PacketGetNetInfoEx((LPTSTR)adapterList2[Open - 1],
			(PULONG)(&ipAddr),(PULONG)(&maskAddr),(PULONG)(&gwAddr),&dhcpFlag);

		if (gwAddr == 0xffffffff)
		{
			printf("\nThe adapter for Outer Network should have Default Gateway information.\n");
			continue;
		}

		break;

	}
	
	int _outer = Open -1;

	while(1)
	{
		printf("\n[Step.2] Select the adapter number for Intra Network: ");
		scanf("%d",&Open);
		
		if (Open == _outer + 1)
		{
			printf("\nThe number has already been selected as Outer Network adapter.\n");
			continue;
		}

		if (Open>AdapterNum) 
		{
			printf("\nThe number must be smaller than %d\n",AdapterNum);
			continue;
		}
		
		if (Open <= 0)
		{
			printf("\nTerminated\n");
			exit(0);
		}

		break;
	}

	int _intra = Open -1;

	//////// check ///// 
	printf("\n");

	WCHAR buffer[1000];
	int j ;

	/* Format and print various data: */
	j =  swprintf( buffer    , L"PSEUD=xxx.xxx.xxx.xxx or dhcp\n");
	j += swprintf( buffer + j, L"OUTERDEV=%s\n", adapterList[_outer]);
	j += swprintf( buffer + j, L"INTRADEV=%s\n", adapterList[_intra]);
	//j += swprintf( buffer + j, L"OUTERDEV=%s\n", adapterList2[_outer]);
	//j += swprintf( buffer + j, L"INTRADEV=%s\n", adapterList2[_intra]);
	//j += swprintf( buffer + j, L"NETWORK=xxx.xxx.xxx.xxx or dhcp\n");
	//j += swprintf( buffer + j, L"GATEWAY=xxx.xxx.xxx.xxx or dhcp\n");

	wprintf(L"%s",buffer);

	printf("\n");
	printf("\"pipmasq.cfg\" template will be created under the current directory.\n");
	printf("\nSave ?[y/other] \n");

	scanf("%s",&Open);

	if ((Open == 'y') || (Open == 'Y'))
	{
		
		char filename[] = "pipmasq.cfg";

		FILE *f;

		if ((f = fopen(filename, "w")) != NULL)
		{
			fwprintf(f,buffer);
			fclose(f);
		}
		printf("Finished successfully....\n");

	}

}

#if 0
	printf("double quotaion = \"\n");   
#endif //0